/******************************************************************************
Every time this Actor is triggered, it chooses one of the given Events randomly
and raises it.

2016-04-30, SeriousBarbie AT googlemail MAKEADOT com
******************************************************************************/

class SBRandomDispatcher extends Triggers;

#exec TEXTURE IMPORT NAME=RandomDispatcher FILE=Textures\RandomDispatcher.pcx MIPS=OFF FLAGS=2

var() name Events[16]; // What Events to call. May even contain gaps.
var() byte EventProbabilities[16]; // The chance of the event to be fired, given in integer percent.
var() byte MaxTriggerCounts[16]; // Maximum number of triggering this event. Set to 0 to use it forever.

var byte Count; // current count of Events

function MoveArraryRow(byte FromIndex, byte ToIndex) {
	Events[ToIndex] = Events[FromIndex];
	Events[FromIndex] = '';
	EventProbabilities[ToIndex] = EventProbabilities[FromIndex];
	MaxTriggerCounts[ToIndex] = MaxTriggerCounts[FromIndex];
}


function CompactArray() {
local byte i, LastEventIndex;

	Count = 0;
	for (i = 0; i < ArrayCount(Events); i++)
	{
		if (Events[i] == '')
		{
			if (GetLastEvent(LastEventIndex))
			{
				if (LastEventIndex > i)
					MoveArraryRow(LastEventIndex, i);
				else
					break; // no Events above me any more
			}
			else
			{
				GotoState(''); // no Events at all left
				break;
			}
		}
		Count++;
	}
}


function bool GetLastEvent(out byte Index) {
/******************************************************************************
Looks for the last event in array. If an event was found, TRUE is returned and
*Index* is set to the index of that event.
******************************************************************************/
local int i;

	for (i = ArrayCount(Events) - 1; i >= 0; i--)
		if (Events[i] != '')
		{
			Index = i;
			return true;
		}
	return false;
}


function PostBeginPlay() {
local byte i;

	// sanity check:
	for (i = 0; i < ArrayCount(Events); i++)
		if (Events[i] != '')
			if (EventProbabilities[i] > 100)
			{
				warn("Probability for event '" $ Events[i] $ "' exceeds valid maximum of 100 - resetting it to 100");
				EventProbabilities[i] = 100;
			}
			else if (EventProbabilities[i] == 0)
			{
				warn("Probability for event '" $ Events[i] $ "' is zero - event removed");
				Events[i] = '';
			}

	CompactArray();
}


auto state Active {

	function Trigger(actor Other, pawn EventInstigator) {
	local byte EventIndex;
	local Actor A;

		Disable('Trigger');
		EventIndex = rand(Count);
		if (rand(100) < EventProbabilities[EventIndex])
		{
			foreach AllActors(class 'Actor', A, Events[EventIndex])
				A.Trigger(Other, EventInstigator);
			if (MaxTriggerCounts[EventIndex] > 0)
				if (MaxTriggerCounts[EventIndex] == 1)
				{
					Events[EventIndex] = '';
					CompactArray();
				}
				else
					MaxTriggerCounts[EventIndex]--;
		}
		Enable('Trigger');
	}
}



defaultproperties {
	Texture=Texture'SBRandomDispatcherV0.RandomDispatcher'
	EventProbabilities(0)=100
	EventProbabilities(1)=100
	EventProbabilities(2)=100
	EventProbabilities(3)=100
	EventProbabilities(4)=100
	EventProbabilities(5)=100
	EventProbabilities(6)=100
	EventProbabilities(7)=100
	EventProbabilities(8)=100
	EventProbabilities(9)=100
	EventProbabilities(10)=100
	EventProbabilities(11)=100
	EventProbabilities(12)=100
	EventProbabilities(13)=100
	EventProbabilities(14)=100
	EventProbabilities(15)=100
}
